@@ -22,7 +22,7 @@
 #  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 #  IN THE SOFTWARE.
 #
-#  $Id: msvs200xprj.py 1302 2009-04-06 19:14:33Z vaclavslavik $
+#  $Id: msvs200xprj.py 1345 2011-02-01 14:02:34Z vaclavslavik $
 #
 #  MS Visual Studio 2003/2005 project files generator script
 #
@@ -78,6 +78,11 @@ optReferencesDefault    = '0'
 optNoReferences         = '1'
 optReferences           = '2'
 
+machineX86              = '1'
+machineARM              = '3'
+machineAMD64            = '17'
+
+
 # ------------------------------------------------------------------------
 #   extended minidom classes to create XML file with specified attributes
 #   order (VC2003 projet file must have attribute "Name" as first)
@@ -210,6 +215,10 @@ def mk_list(list):
     # remove empty items from the list:
     return ";".join([x for x in list.split(";") if len(x) > 0])
 
+def mk_listsp(list):
+    # remove empty items from the list:
+    return ";".join([x for x in list.split(" ") if len(x) > 0])
+
 def bool2vcstr(value):
     """Returns "true"/"false" or "TRUE"/"FALSE" string depending on format
        version."""
@@ -321,6 +330,11 @@ class ProjectGeneratorMsvc9:
         cfg = configs[config][0]['MSVS_PLATFORM']
         return cfg == 'pocketpc2003'
 
+    def isX64Config(self, config):
+        """Returns true if given config targets a 64bit system."""
+        cfg = configs[config][0]['MSVS_PLATFORM']
+        return cfg == 'win64'
+
 
     def assignGUIDs(self, sln_targets):
         """Returns the dictionary containing GUIDs indexed by target ids."""
@@ -554,7 +568,7 @@ Microsoft Visual Studio Solution File, Format Version 10.00
         tool.setAttribute("AdditionalIncludeDirectories", mk_list(cfg._include_paths))
         tool.setAttribute("PreprocessorDefinitions", mk_list(cfg._defines))
 
-        if cfg._optimize == "0" and cfg._debug == '1':
+        if cfg._optimize == "0" and cfg._rtl_dbg == 'on':
             # VC2008 projects use /MP switch, which is incompatible with
             # MinimalRebuild; /MP is more useful, so omit MinimalRebuild:
             if _MSVS_VCPROJ_VERSION in ["7.10", "8.00"]:
@@ -621,11 +635,17 @@ Microsoft Visual Studio Solution File, Format Version 10.00
                     # automatic PCH support (/YX option) was removed in VC8, so
                     # disable the use of PCH completely when this option is
                     # specified (what else can we do?)
+                    #
+                    # FIXME: instead, we could pick a source file that includes
+                    #        _pch_header (if specified) at random and use that;
+                    #        or at least issue a warning
                     do_use_pch = False
 
             if do_use_pch:
                 tool.setAttribute("PrecompiledHeaderThrough", cfg._pch_header)
-                tool.setAttribute("PrecompiledHeaderFile", cfg._pch_file)
+                if cfg._pch_file:
+                    tool.setAttribute("PrecompiledHeaderFile",
+                                      "%s\\%s" % (cfg._builddir, cfg._pch_file))
 
 
         tool.setAttribute("ObjectFile", "%s\\%s\\" % (cfg._builddir, t.id) )
@@ -662,7 +682,7 @@ Microsoft Visual Studio Solution File, Format Version 10.00
         tool.setAttribute("AdditionalDependencies", ldlibs)
         tool.setAttribute("OutputFile", "%s%s" % (cfg._targetdir, cfg._targetname))
 
-        if cfg._debug == '1':
+        if cfg._rtl_dbg == 'on':
             tool.setAttribute("LinkIncremental", linkIncrementalYes)
         else:
             tool.setAttribute("LinkIncremental", linkIncrementalNo)
@@ -688,13 +708,22 @@ Microsoft Visual Studio Solution File, Format Version 10.00
                              "%s" % self.app_type_code[cfg._type_nick])
 
         if self.isEmbeddedConfig(c):
-            tool.setAttribute("TargetMachine", "3")
+            tool.setAttribute("TargetMachine", machineARM)
             tool.setAttribute("DelayLoadDLLs", "$(NOINHERIT)")
-            if cfg._debug == '0':
+            if cfg._rtl_dbg == 'off':
                 tool.setAttribute("OptimizeReferences", optReferences)
                 tool.setAttribute("EnableCOMDATFolding", "2")
         else:
-            tool.setAttribute("TargetMachine", "1")
+            if self.isX64Config(c):
+                tool.setAttribute("TargetMachine", machineAMD64)
+            else:
+                tool.setAttribute("TargetMachine", machineX86)
+            # optimize linking unless we're using debug RTL (in which case we
+            # don't build production version anyhow) or we don't use /debug at
+            # all (in which case it's optimized by default)
+            if cfg._rtl_dbg == 'off' and cfg._debug == '1':
+                tool.setAttribute("OptimizeReferences", optReferences)
+                tool.setAttribute("EnableCOMDATFolding", "2")
 
 
     def buildLibrarianToolElement(self, tool, prjname, cfg, c, t):
@@ -709,7 +738,21 @@ Microsoft Visual Studio Solution File, Format Version 10.00
 
 
     def buildIdlToolElement(self, tool, prjname, cfg, c, t):
-        tool.setAttribute("PreprocessorDefinitions", mk_list(cfg._defines))
+        # MIDL compiler has a bug in it that prevents quoted values in
+        # PreprocessorDefinitions from being passed correctly to the compiler.
+        # We work around it by using AdditionalOptions.
+        # See http://www.vtk.org/Bug/view.php?id=8165 for the gory details.
+        all_defines = [x for x in cfg._defines.split(";") if len(x) > 0]
+        extras = []
+        simple_defines = []
+        for d in all_defines:
+            if '"' in d:
+                extras.append("/D %s" % d)
+            else:
+                simple_defines.append(d)
+        if extras:
+            tool.setAttribute("AdditionalOptions", " ".join(extras))
+        tool.setAttribute("PreprocessorDefinitions", ";".join(simple_defines))
         tool.setAttribute("AdditionalIncludeDirectories", mk_list(cfg._include_paths))
 
 
@@ -750,7 +793,6 @@ Microsoft Visual Studio Solution File, Format Version 10.00
         toolHandlers = {
                          'VCBscMakeTool'    : self.buildBscMakeToolElement,
                          'VCCLCompilerTool' : self.buildCompilerToolElement,
-                         'VCIDLTool'        : self.buildIdlToolElement, 
                          'VCMIDLTool'       : self.buildIdlToolElement, 
                          'VCLibrarianTool'  : self.buildLibrarianToolElement,
                          'VCLinkerTool'     : self.buildLinkerToolElement,
@@ -782,16 +824,12 @@ Microsoft Visual Studio Solution File, Format Version 10.00
                         'VCAuxiliaryManagedWrapperGeneratorTool',
                     ]
         else:
-            if _MSVS_VCPROJ_VERSION == "8.00":
-                idlTool = "VCMIDLTool"
-            else:
-                idlTool = "VCIDLTool"
             tools = [
                         'VCPreBuildEventTool',
                         'VCCustomBuildTool',
                         'VCXMLDataGeneratorTool',
                         'VCWebServiceProxyGeneratorTool',
-                        idlTool,
+                        'VCMIDLTool',
                         'VCCLCompilerTool',
                         'VCManagedResourceCompilerTool',
                         'VCResourceCompilerTool',
@@ -859,7 +897,11 @@ Microsoft Visual Studio Solution File, Format Version 10.00
             elif deps.startswith('$(SOURCE) '):
                 deps = deps[len('$(SOURCE) '):]
 
-            el.setAttribute('AdditionalDependencies', deps)
+            if _MSVS_SLN_VERSION == "10.00":
+                el.setAttribute('AdditionalDependencies', mk_listsp(deps))
+            else:
+                el.setAttribute('AdditionalDependencies', deps)
+
         return el
 
 
