@@ -21,7 +21,7 @@
 #  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 #  IN THE SOFTWARE.
 #
-#  $Id: utils.py 1302 2009-04-06 19:14:33Z vaclavslavik $
+#  $Id: utils.py 1319 2009-10-04 22:05:21Z vaclavslavik $
 #
 #  Misc utility functions for use in Bakefiles
 #
@@ -321,7 +321,9 @@ def makeUniqueName(name, all):
         conflicts = len([n for n in all if n.endswith(x)])
         if conflicts == 1: # only this one
             return x.replace(dirsep, '_')
-    raise errors.Error("don't know how to create object file name for \"%s\"" % name)
+
+    # as the last resort, use full path - ugly, but works:
+    return '_'.join(split).replace('..','__')
 
 
 allObjectsBasenames = {}
@@ -401,7 +403,7 @@ def sources2objects(sources, target, ext, objSuffix=''):
         allObjectsBasenames[target] = {}
     basenames = allObjectsBasenames[target]
     dirsep = mk.vars['DIRSEP']
-    for s in getPossibleValues(sources):
+    for s in getPossibleValues(sources2):
         full = s[:s.rfind('.')]
         base = full[full.rfind(dirsep)+1:]
         if base not in basenames:
@@ -648,6 +650,39 @@ def addPrefixToList(prefix, value):
     return substitute2(value, lambda c,s: callback(prefix,c,s))
 
 
+
+
+def getTagAttrib(name, default):
+    """
+        Returns the value of the attribute 'name' for the current tag which is
+        being processed.  If the tag's attribute has not been provided, then
+        the default value is returned.
+
+        With this function you can define own tags which accept new attributes.
+
+        E.g.
+
+             <define-tag name="mytag" rules="exe,dll,lib">
+                <define>$(value)</define>
+                <include>$(getTagAttrib('override', 'default'))</include>
+             </define-tag>
+
+             <exe id="myexe">
+                <!-- defines the 'A' symbol and includes the 'default' path -->
+                <mytag>A</mytag>
+
+                <!-- defines the 'A' symbol and includes the 'custom' path -->
+                <mytag override="custom">A</mytag>
+             </exe>
+    """
+    if 'attributes' not in mk.__curNamespace:
+        return default
+    attr = mk.__curNamespace['attributes']
+    if name in attr:
+        return attr[name]
+    else:
+        return default
+
 def mkPathPrefix(p):
     if p == '.':
         return ''
